REM This file contains all the functions that are involved in the process of tracking video stream

REM function initData
REM this function is used to get initial state of Springstreams and Stream object at the start of measurement

function KMA_initData(SpStr as object,Stream1 as object)
    m.SpringStreams = SpStr
    m.clock = CreateObject("roDateTime")
    m.clock1 = CreateObject("roTimeSpan")
    m.pause=0
    m.closed = false
    m.finished = false
    m.KMAPlaycount = 0
    KMA_update(Stream1)
    'KMA_sync(stream1)
    m.streamm=Stream1
    m.posTimer = m.top.findNode("positionTimer")
    m.syncTimer = m.top.findNode("syncTimer")
    if m.posTimer.control = "none"
    m.posTimer.ObserveField("fire","update")
    end if
    if m.syncTimer.control = "none"
    m.syncTimer.ObserveField("fire","sync")
    end if
    m.posTimer.control = "start"
    m.syncTimer.control = "start"
   
end function

sub update()
KMA_update(m.streamm)
end sub

sub sync()
KMA_sync(m.streamm)
end sub

REM function KMA_update
Rem this function is called every 200ms to track playhead of the stream that currently is being tracked

function KMA_update(Streamm as object) as object
    if (Streamm.closed = true)
        return 0
    end if
    m.lasttime = Streamm.state[1]
    m.now = m.stAdapter.getPosition()
    if(m.now =0)
    m.now =Cint(Streamm.KMAvideo.position)
    end if
    if (m.lasttime = m.now and m.lasttime <>0)
        'pause? sync the stream immediately
        if (m.pause >= 0)
            m.pause++
            if (m.pause = Streamm.collector.pausesync)
                Streamm.doSync = true
            end if
            if (m.pause >= Streamm.collector.pausesync)
                if Streamm.KMAvideo.state= "finished" and m.finished =false
                    m.finished = true
                    KMA_sync(Streamm)
                end if
            end if
        end if
        return 0
    end if
    if (m.pause >= Streamm.collector.pausesync)
        if Streamm.KMAvideo.state <> "buffering"
            m.KMAPlaycount ++
            if(m.KMAPlaycount >1)
                m.kmaplaycount =0
                KMA_addState(m.now,Streamm)
            end if
        end if
    else if (m.lasttime < m.now - 1)
        '// forward
        if Streamm.KMAvideo.state <> "buffering"
            m.KMAPlaycount ++
            if(m.KMAPlaycount >1)
                m.kmaplaycount =0
                KMA_addTempState(m.now,Streamm)
            end if
        end if

    else if (m.lasttime > m.now)
        '// backward
        if Streamm.KMAvideo.state <> "buffering"
            m.KMAPlaycount ++
            if(m.KMAPlaycount >1)
                m.kmaplaycount =0
                KMA_addTempState(m.now,Streamm)
            end if
        end if

    else
        '// linear
        Streamm.state[1] = m.now
        if (Streamm.state[1] - Streamm.state[0] >= Streamm.syncrate)
            if (m.now - Streamm.lastSync >= Streamm.syncrate)
                Streamm.doSync = true
                KMA_switchSyncRate(Streamm)
            end if
        end if
        m.pause = 0
        m.streamm=Streamm
        Streamm = invalid

    end if
end function

function KMA_encrypt(id as string)
    ba2 = CreateObject("roByteArray")
    ba2.FromAsciiString(id)
    digest = CreateObject("roEVPDigest")
    digest.Setup("md5")
    digest.Update(ba2)
    result = digest.Final()
    return Left(result,16)
end function

REM function getKID()
REM returns kid from registry if already available or create a new one
sub getKID() as string
    deviceInfo = CreateObject("roDeviceInfo")
    kid = ""
    kidRegistry = createObject("roRegistrySection","kid")
    if kidRegistry.Exists("kid")
    kid = kidRegistry.Read("kid")
    else
    kid = UCase(deviceInfo.GetRandomUUID())
    kidRegistry.Write("kid",kid)
    end if   
    return kid
end sub

REM function getRokuVersion()
REM returns current os version of device
sub getRokuVersion() as string
    deviceInfo = CreateObject("roDeviceInfo")
    osv=""
    deviceVersion =  deviceInfo.getOSVersion()
    osv = deviceVersion.major+"."+deviceVersion.minor
    return osv
end sub

REM function sync
REM this function is called by timer to synchronize data that is collected
function KMA_sync( Streamm as object)

    if Streamm.doSync = true
        app = CreateObject("roAppInfo")
        deviceInfo = CreateObject("roDeviceInfo")
        deviceversion = getRokuVersion()
        kid = KMA_encrypt(getKID())
        Streamm.attributes["pst"] = Streamm.playStates
        Streamm.attributes["vt"] = m.clock1.TotalSeconds()
        Streamm.attributes["uid"] = Streamm.id
        if(m.clock1.TotalSeconds() > 0 and (Streamm.state[1]>0 or Streamm.playStates.count()>1))
            Streamm.attributes["dur"] = Streamm.stAdapter.getDuration()
            if(Streamm.stAdapter.getDuration() > 0)
            Streamm.attributes["dur"] = Streamm.KMAVideo.duration
            end if
            else
            Streamm.attributes["dur"] = 0
        end if
        meta = CreateObject("roAssociativeArray")
        if(Streamm.stAdapter.doesExist("getMeta"))
            meta = Streamm.stAdapter.getMeta
        else
            meta = Streamm.getMetaDefault
        end if
        meta["kid"] = kid
        meta["rid"] = KMA_encrypt(deviceInfo.GetChannelClientId())
        if(deviceInfo.IsRIDADisabled() = false)
            meta["rai"] = KMA_encrypt(deviceinfo.GetRIDA())
        end if
        meta["v"] = "R " +Streamm.version
        meta["os"]="ROKU"
        meta["osv"]=deviceversion
        meta["an"]=app.getTitle()
        meta["av"]="1.0.0"
        KMA_send(meta, Streamm.attributes)
        Streamm.lastSync = Streamm.state[1]
    end if
    Streamm.doSync = false
    m.streamm=Streamm
    Streamm = invalid
end function

REM function KMA_switchsyncRates
function KMA_switchSyncRate(Streamm as object)
    KMA_sync( Streamm)
    if (Streamm.syncrates <> invalid)
        if Streamm.syncratesIndex < Streamm.syncrates.count()
            Streamm.syncrate = Streamm.syncrates[Streamm.syncratesIndex]
            Streamm.syncratesIndex = Streamm.syncratesIndex +1
            m.streamm=Streamm
            Streamm = invalid
            'collector.debug("switch syncrate to " + Streamm.syncrate)
        end if
    end if
end function


REM function KMA_addplaystate
REM this function adds playstate to array whenever there is action performed by user or whenever sync delay is triggered
function KMA_addPlayState(v,Streamm)
    if (Streamm.playStates.count() < Streamm.collector.maxstates)
        Streamm.playStates.push(v)
    end if
    m.streamm=Streamm
    Streamm = invalid
end function

function KMA_addState(now,Streamm)
    m.clock.mark()
    clock = CreateObject("roDateTime")
    Streamm.state = [now, now, strI(clock.AsSeconds(),36)] ' base 36 conversion
    KMA_addPlayState(Streamm.state,Streamm)
    Streamm.doSync = true
    m.pause = 0
    m.streamm=Streamm
    Streamm = invalid
end function


'/**
'* @method addTempState
'* @description this functions checks for user crubbing/winding operation before adding playstate
'*/
rem checking playstate for scrubbing/winding

function KMA_addTempState(temp,Streamm)
    currPos = Streamm.stAdapter.getPosition()
    if currPos = 0
        currPos =  Cint(Streamm.KMAvideo.position)
    end if
    if temp = currPos or temp + 1 = currPos
        Streamm.tempStates++
    else
        Streamm.tempStates = 0
    end if
    if (Streamm.tempStates >= 5)
        finalState =0
        if (temp = currPos)
            finalState = temp
        else
            finalState = temp + 1
        end if
        Streamm.tempStates = 0
        if (finalState - Streamm.state[1] >= 1 or Streamm.state[1] - finalState >= 1)
            KMA_addState(temp,Streamm)
        end if
    end if
    m.streamm=Streamm
    Streamm = invalid
end function





REM function KMA_findsite
REM returns site name

function KMA_findSite(v) as string
    if (v<> invalid and v.site <> invalid) return v.site
        return m.SpringStreams.defaultSite
    end function


    function KMA_debug(v) as void
    end function

    REM function pr
    REM returns protocol

    function KMA_pr() as string
        return "https://"
    end function

    REM Sync and stop the stream tracking immediately.
    REM @method stop
    REM @description Sync and stop the stream tracking immediately.
    function KMA_stop(Springstr,Streamm)
        m.springstreams = Springstr
	m.posTimer.control = "stop"
        m.syncTimer.control = "stop"
        if (streamm.closed = false)
            streamm.doSync = true
            streamm.closed = true
            m.closed=true
            KMA_sync(streamm)
            Streamm = invalid
        end if
    end function
    REM This function creates and encodes the request
    function KMA_sf(v) as string
        a = []
        if type(v)="roString" or type(v) = "String"
            return v.EncodeUriComponent()
        else if type(v) = "roInteger"
            return v.toStr()
        else if type(v) = "roInteger" or type(v) = "roInt" or type(v) = "roFloat" or type(v) ="Double"
            return v.tostr()
        else if type(v) = "roArray"
            if (v = invalid)
                return "null"
            end if
            'if (type(v.count()) = "number" )
            l = v.count()
            for  i= 0 to l-1 step 1
                a.push(KMA_sf(v[i]))
            end for

            return "," + a.join("+") + ";"
        else if type(v) ="roAssociativeArray"
            for each k in v
                if type(k) = "roString"
                    if (k <> "site")
                        d = KMA_sf(v[k])
                        if (d <> invalid)
                            a.push(KMA_sf(k) + "=" + d)
                        end if
                    end if
                end if
            end for
            return "," + a.join("+") + ";"
        end if
        return ""
    end function

    function KMA_sencode (meta, v) as object
        return KMA_sf([
            [
                meta,
                v
            ]
        ])
    end function

    REM this function invokes the sendRequest task and sends request as field to task interface.
    function KMA_send(meta, what) as void
        if m.springstreams.tracking = true
            if (type(what.stream) = invalid or what.stream = "") then
                print "Stream is mandatory"
            else
                clock = CreateObject("roDateTime")
                s = KMA_pr() + KMA_findSite(what) + "." + m.SpringStreams.domain + "/j0=" + KMA_sencode(meta, what) + "?lt=" + strI((clock.AsSeconds()), 36).tostr()
                'call task
                m.requestTask = CreateObject("roSGNode", "KMARequest")
                m.requestTask.setField("request", s)
                m.requestTask.setField("debug", m.SpringStreams.debug)
                m.requesttask.control = "RUN"
            end if
        end if
    end function
