REM     * @memberof SpringStreams
REM     * @description This is main function/constructor that user calls when he wants to initialize the sensor.
REM     *
REM     * @param site
REM     *            {String} stream The name of the stream content


function SpringStreams(site as string) as object
    init =sub()
    end sub
    m.clockS = CreateObject("roDateTime")
    m.version = "1.4.4"
    m.domain = "2cnt.net"
    m.collectorDelay = 200
    m.syncDelay = 2000
    m.defaultSite = "default"
    if site<>invalid m.defaultSite = site
        m.streamdata = createObject("roArray",50,true)
        m.syncrates = createObject("roArray",50,true)
        m.namespace = "com.kma.springstreams"
        ' synchronize ever 20 seconds of linear video
        m.syncrate = 20
        ' after 1 second of pause (6 events of the timecollect timer means bit more than one second) synchronize data
        m.pausesync = 6
        ' define the maximum amount of states to be tracked
        m.maxstates=50
        m.debug = false
        m.tracking = true
            m.nextUid =""
        m.config={}

        m.KMAdoUpdate = function()
            for each stream1 in m.streamdata
                KMA_update(stream1)
            end for
        end function

        m.KMAdoSync =function()
            for each stream1 in m.streamdata
                KMA_sync(stream1)
            end for
        end function
        m.KMAdoUpdate()
        m.KMAdoSync()
        m.generateNextUid = function()
            randomUidPart = stri(Int(rnd(4000000000) + m.clock.GetMilliseconds()),36)
            millisUidPart = strI((m.clockS.AsSeconds() + m.clock.GetMilliseconds()),36)
            return millisUidPart + randomUidPart + strI(m.clock.GetMilliseconds(),36)
        end function

        REM function initTimers
        REM this function is called immediately after sensor is initialized
        REM this initializes the timerTask

        m.initTimers = function(stream1,att,adapter)
          KMA_initData(m,m.streamdata[0])
        end function

        m.configureSyncRates =function(arr)
            if (type(arr)<>invalid)
                m.syncrates = arr
            end if
        end function

        m.getSyncRates =function() as object
            return m.syncrates
        end function
        getConfig =function() as object
            config = {
                "2cnt.net": {

                    "syncrates": [3, 7, 10, 10, 10, 10, 10, 60]
                }
            }
            m.config = config
            return config
        end function

        configure = function(config) as void
            domainconfig={}
            if (type(config)<>invalid)
                for each att in config
                    'print att
                    if (att = m.domain)
                        domainConfig = config[att]
                    end if
                end for
                if (type(domainConfig)<>invalid)
                    m.configureSyncRates(domainConfig.syncrates)
                end if
            end if
        end function
        configure(getconfig())


        REM     * @memberOf SpringStreams
        REM     * @method setTracking
        REM     * @description Call this method when user wish to stop the measurement of streams.
        REM     * @param trackingEnabled {boolean} By default this will be true, If user wish to stop the measurement
        REM     *                        He can call this method and send false as parameter

        m.setTracking = function(trackingEnabled)
            m.tracking = trackingEnabled
        end function

        REM        * @memberOf SpringStreams
        REM        * @method isTracking
        REM        * Delivers the value <code>true</code> when the tracking
        REM        * is activated otherwise the value is <code>false</code>.

        m.isTracking = function() as boolean
            return m.tracking
        end function
        
       
         REM     * @memberof SpringStreams
         REM     * @method getNextUid
         REM     * @description Call this method to retrieve nextUid ahead of measurement
         REM     * @returns nextUid

        m.getNextUid = function() as string
              m.nextUid = m.generateNextUid()
              return m.nextUid
        end function

        REM     * @memberof SpringStreams
        REM     * @method track
        REM     * @description Call this method to start the tracking of a streaming
        REM     *              content. The sensor gets access to the stream by the given
        REM     *              adapter. The variable <code>stream</code> is mandatory in
        REM     *              the attributes object. the Stream can be described by the
        REM     *              atts parameter.
        REM     *
        REM     * @param stream
        REM     *            {String} stream The name of the stream content
        REM     * @param att
        REM     *            {Map} att A map which contains information about the streaming
        REM     *            content.
        REM     * @param adapter
        REM     *            {StreamAdapter} adapter The stream adapter which handles the
        REM     *            access to the underlying streaming content.
        REM     * @returns A StreamClass object.


        m.track =function(stream1, att, adapter) as object
            m.ret =  StreamClass(m, stream1, att, adapter)
            m.streamdata.push(m.ret) ''// SET
            m.inittimers(stream1,att,adapter)
            return m.ret
        end function

        REM     * @memberof SpringStreams
        REM     * @method trackWithUid
        REM	    * @description new track method for barb with additional parameter handle
        REM	    * @param handle The UID generated when sensor starts and is sent back to customer before starting measurement

        m.trackWithUid = function(stream1, att, adapter,handle)as object
            if(handle<>m.nextuid)
                return "Invalid UID"
            else
                return m.track(stream1, att, adapter)
            end if
        end function

        REM     * @memberof SpringStreams
        REM     * @method unload
        REM     * @description External callback, synchronize and unload all the open streams.
        REM     *
        REM     * @returns A SpringStreamsClass object.


        m.unload = function()

           KMA_stop(m,m.streamdata[0])
        end function

        return m


    end function




